@@ -1,8 +1,8 @@
 #include "xdo_cmd.h"
 
-int cmd_get_num_desktops(int argc, char **args) {
+int cmd_get_num_desktops(context_t *context) {
   int ret = 0;
-  char *cmd = *args;
+  char *cmd = context->argv[0];
   long ndesktops = 0;
 
   int c;
@@ -13,10 +13,12 @@ int cmd_get_num_desktops(int argc, char **args) {
   static const char *usage = "Usage: %s\n";
   int option_index;
 
-  while ((c = getopt_long_only(argc, args, "h", longopts, &option_index)) != -1) {
+  while ((c = getopt_long_only(context->argc, context->argv, "+h",
+                               longopts, &option_index)) != -1) {
     switch (c) {
       case 'h':
         printf(usage, cmd);
+        consume_args(context, context->argc);
         return EXIT_SUCCESS;
         break;
       default:
@@ -25,15 +27,14 @@ int cmd_get_num_desktops(int argc, char **args) {
     }
   }
 
-  argc -= optind;
-  args += optind;
+  consume_args(context, optind);
 
-  if (argc != 0) {
-    fprintf(stderr, usage, cmd);
-    return 1;
-  }
+  //if (context->argc != 0) {
+    //fprintf(stderr, usage, cmd);
+    //return 1;
+  //}
 
-  ret = xdo_get_number_of_desktops(xdo, &ndesktops);
+  ret = xdo_get_number_of_desktops(context->xdo, &ndesktops);
 
   printf("%ld\n", ndesktops);
   return ret;
