@@ -1,9 +1,8 @@
 #include "xdo_cmd.h"
 
-int cmd_windowfocus(int argc, char **args) {
+int cmd_windowfocus(context_t *context) {
   int ret = 0;
-  Window wid;
-  char *cmd = *args;
+  char *cmd = *context->argv;
   int opsync = 0;
 
   int c;
@@ -16,15 +15,17 @@ int cmd_windowfocus(int argc, char **args) {
     { 0, 0, 0, 0 },
   };
   static const char *usage = 
-    "Usage: %s wid\n"
+    "Usage: %s [window=%1]\n"
     "--sync    - only exit once the window has focus\n";
 
   int option_index;
-  while ((c = getopt_long_only(argc, args, "h", longopts, &option_index)) != -1) {
+  while ((c = getopt_long_only(context->argc, context->argv, "+h",
+                               longopts, &option_index)) != -1) {
     switch (c) {
       case 'h':
       case opt_help:
         printf(usage, cmd);
+        consume_args(context, context->argc);
         return EXIT_SUCCESS;
         break;
       case opt_sync:
@@ -36,24 +37,26 @@ int cmd_windowfocus(int argc, char **args) {
     }
   }
 
-  argc -= optind;
-  args += optind;
+  consume_args(context, optind);
 
-  if (argc != 1) {
+  const char *window_arg = "%1";
+  if (!window_get_arg(context, 0, 0, &window_arg)) {
     fprintf(stderr, usage, cmd);
-    return 1;
+    return EXIT_FAILURE;
   }
 
-  wid = (Window)strtol(args[0], NULL, 0);
-  ret = xdo_window_focus(xdo, wid);
-  if (ret) {
-    fprintf(stderr, "xdo_window_focus reported an error\n");
-  } else {
-    if (opsync) {
-      xdo_window_wait_for_focus(xdo, wid, 1);
+  window_each(context, window_arg, {
+    ret = xdo_window_focus(context->xdo, window);
+    if (ret) {
+      fprintf(stderr, "xdo_window_focus reported an error\n");
+      return ret;
+    } else {
+      if (opsync) {
+        xdo_window_wait_for_focus(context->xdo, window, 1);
+      }
     }
-  }
-  
+  }); /* window_each(...) */
+
   return ret;
 }
 
