@@ -209,39 +209,54 @@ int xdo_window_move(const xdo_t *xdo, Window wid, int x, int y) {
   return _is_success("XConfigureWindow", ret == 0);
 }
 
-int xdo_window_setsize(const xdo_t *xdo, Window wid, int width, int height, int flags) {
+int xdo_window_translate_with_sizehint(const xdo_t *xdo, Window window,
+                                       int width, int height, int *width_ret,
+                                       int *height_ret) {
+  XSizeHints hints;
+  long supplied_return;
+  XGetWMNormalHints(xdo->xdpy, window, &hints, &supplied_return);
+  if (supplied_return & PResizeInc) {
+    width *= hints.width_inc;
+    height *= hints.height_inc;
+  } else {
+    fprintf(stderr, "No size hints found for window %ld\n", window);
+    *width_ret = width;
+    *height_ret = width;
+  }
+
+  if (supplied_return & PBaseSize) {
+    width += hints.base_width;
+    height += hints.base_height;
+  }
+
+  *width_ret = width;
+  *height_ret = height;
+
+  return XDO_SUCCESS;
+}
+
+int xdo_window_setsize(const xdo_t *xdo, Window window, int width, int height, int flags) {
   XWindowChanges wc;
   int ret = 0;
   int cw_flags = 0;
 
-  wc.width = width;
-  wc.height = height;
-
   if (flags & SIZE_USEHINTS) {
-    XSizeHints hints;
-    long supplied_return;
-    memset(&hints, 0, sizeof(hints));
-    XGetWMNormalHints(xdo->xdpy, wid, &hints, &supplied_return);
-    if (supplied_return & PResizeInc) {
-      wc.width *= hints.width_inc;
-      wc.height *= hints.height_inc;
-    } else {
-      fprintf(stderr, "No size hints found for this window\n");
-    }
-
-    if (supplied_return & PBaseSize) {
-      wc.width += hints.base_width;
-      wc.height += hints.base_height;
-    }
-
+    xdo_window_translate_with_sizehint(xdo, window, width, height,
+                                       &wc.width, &wc.height);
+  } else {
+    wc.width = width;
+    wc.height = height;
   }
 
-  if (width > 0)
+  if (width > 0) {
     cw_flags |= CWWidth;
-  if (height > 0)
-    cw_flags |= CWHeight;
+  }
 
-  ret = XConfigureWindow(xdo->xdpy, wid, cw_flags, &wc);
+  if (height > 0) {
+    cw_flags |= CWHeight;
+  }
+
+  ret = XConfigureWindow(xdo->xdpy, window, cw_flags, &wc);
   XFlush(xdo->xdpy);
   return _is_success("XConfigureWindow", ret == 0);
 }
@@ -291,6 +306,28 @@ int xdo_window_focus(const xdo_t *xdo, Window wid) {
   return _is_success("XSetInputFocus", ret == 0);
 }
 
+int xdo_window_wait_for_size(const xdo_t *xdo, Window window, unsigned int width,
+                             unsigned int height, int flags, int to_or_from) {
+  unsigned int cur_width, cur_height;
+
+  if (flags & SIZE_USEHINTS) {
+    xdo_window_translate_with_sizehint(xdo, window, width, height,
+                                       &width, &height);
+  }
+
+  xdo_get_window_size(xdo, window, (unsigned int *)&cur_width,
+                      (unsigned int *)&cur_height);
+  while (to_or_from == SIZE_TO
+         ? (cur_width != width && cur_height != height)
+         : (cur_width == width && cur_height == height)) {
+    xdo_get_window_size(xdo, window, (unsigned int *)&cur_width,
+                        (unsigned int *)&cur_height);
+    usleep(30000);
+  }
+
+  return 0;
+}
+
 int xdo_window_wait_for_active(const xdo_t *xdo, Window window, int active) {
   Window activewin = 0;
   int ret = 0;
@@ -575,9 +612,16 @@ int xdo_window_raise(const xdo_t *xdo, Window wid) {
 
 int xdo_mousemove(const xdo_t *xdo, int x, int y, int screen)  {
   int ret = 0;
-  ret = XTestFakeMotionEvent(xdo->xdpy, screen, x, y, CurrentTime);
+
+  /* There is a bug (feature?) in XTestFakeMotionEvent that causes
+   * the screen number in the request to be ignored. The internets
+   * seem to recommend XWarpPointer instead, ie;
+   * https://bugzilla.redhat.com/show_bug.cgi?id=518803
+   */
+  Window screen_root = RootWindow(xdo->xdpy, screen);
+  ret = XWarpPointer(xdo->xdpy, None, screen_root, 0, 0, 0, 0, x, y);
   XFlush(xdo->xdpy);
-  return _is_success("XTestFakeMotionEvent", ret == 0);
+  return _is_success("XWarpPointer", ret == 0);
 }
 
 int xdo_mousemove_relative_to_window(const xdo_t *xdo, Window window, int x, int y) {
@@ -767,8 +811,8 @@ int xdo_type(const xdo_t *xdo, Window window, char *string, useconds_t delay) {
 
     //_xdo_send_key(xdo, window, keycode, modstate, True, delay);
     //_xdo_send_key(xdo, window, keycode, modstate, False, delay);
-    xdo_keysequence_list_do(xdo, window, &key, 1, True, NULL, delay);
-    xdo_keysequence_list_do(xdo, window, &key, 1, False, NULL, delay);
+    xdo_keysequence_list_do(xdo, window, &key, 1, True, NULL, delay / 2);
+    xdo_keysequence_list_do(xdo, window, &key, 1, False, NULL, delay / 2);
 
     /* XXX: Flush here or at the end? or never? */
     //XFlush(xdo->xdpy);
@@ -888,8 +932,8 @@ int xdo_keysequence(const xdo_t *xdo, Window window, const char *keyseq,
                     useconds_t delay) {
   int ret = 0;
   int modifier = 0;
-  ret += _xdo_keysequence_do(xdo, window, keyseq, True, &modifier, delay);
-  ret += _xdo_keysequence_do(xdo, window, keyseq, False, &modifier, delay);
+  ret += _xdo_keysequence_do(xdo, window, keyseq, True, &modifier, delay / 2);
+  ret += _xdo_keysequence_do(xdo, window, keyseq, False, &modifier, delay / 2);
   return ret;
 }
 
@@ -912,7 +956,7 @@ int xdo_window_get_focus(const xdo_t *xdo, Window *window_ret) {
   return _is_success("XGetInputFocus", ret == 0);
 }
 
-int xdo_window_wait_for_focus(const xdo_t *xdo, Window wid, int want_focus) {
+int xdo_window_wait_for_focus(const xdo_t *xdo, Window window, int want_focus) {
   Window focuswin = 0;
   int ret;
   ret = xdo_window_get_focus(xdo, &focuswin);
@@ -920,7 +964,8 @@ int xdo_window_wait_for_focus(const xdo_t *xdo, Window wid, int want_focus) {
     return ret;
   }
 
-  while (want_focus ? focuswin != wid : focuswin == wid) {
+  while (want_focus ? focuswin != window : focuswin == window) {
+    //printf("Current win: %ld, want %ld\n", focuswin, window);
     usleep(30000); /* TODO(sissel): Use exponential backoff up to 1 second */
     ret = xdo_window_get_focus(xdo, &focuswin);
     if (ret != 0) {
@@ -1333,7 +1378,7 @@ void _xdo_send_key(const xdo_t *xdo, Window window, charcodemap_t *key,
             case Mod3Mask: maskname = "mod3"; break;
           }
           modkey = _xdo_cached_modifier_to_keycode(xdo, masks[i]);
-          //printf("Sending modkey for mod %d, %s, (keycode %d) %s\n",
+          //fprintf(stderr, "Sending modkey for mod %d, %s, (keycode %d) %s\n",
                  //masks[i], maskname, modkey, is_press ? "down" : "up");
           XTestFakeKeyEvent(xdo->xdpy, modkey, is_press, CurrentTime);
           XSync(xdo->xdpy, False);
@@ -1435,7 +1480,14 @@ int xdo_active_keys_to_keycode_list(const xdo_t *xdo, charcodemap_t **keys,
   for (keycode = xdo->keycode_low; keycode <= xdo->keycode_high; keycode++) {
     if ((keymap[(keycode / 8)] & (1 << (keycode % 8))) \
         && _xdo_cached_keycode_to_modifier(xdo, keycode)) {
-      /* This keycode is active and is a modifier */
+      /* This keycode is active and is a modifier, record it. */
+
+      /* Zero the charcodemap_t entry before using it.
+       * Fixes a bug reported by Hong-Leong Ong - where 
+       * 'xdotool key --clearmodifiers ...' sometimes failed trying
+       * to clear modifiers that didn't exist since charcodemap_t's modmask was
+       * uninitialized */
+      memset(*keys + *nkeys, 0, sizeof(charcodemap_t));
 
       (*keys)[*nkeys].code = keycode;
       (*nkeys)++;
@@ -1507,7 +1559,8 @@ int xdo_clear_active_modifiers(const xdo_t *xdo, Window window, xdo_active_mods_
   return ret;
 }
 
-int xdo_set_active_modifiers(const xdo_t *xdo, Window window, const xdo_active_mods_t *active_mods) {
+int xdo_set_active_modifiers(const xdo_t *xdo, Window window, const
+                             xdo_active_mods_t *active_mods) {
   int ret = 0;
   xdo_keysequence_list_do(xdo, window, active_mods->keymods,
                           active_mods->nkeymods, True, NULL, DEFAULT_DELAY);
